
% Deflate implementation for Haskell

% Copyright 2004 Ian Lynagh <igloo@earth.li>
% Licence: 3 clause BSD.

% XXX All comments/docs may be wrong

\section{Deflate}

This module provides a Haskell implementation of the deflate function,
as described by RFC 1951.

\begin{code}
module Data.Compression.Deflate.Deflate (deflate) where

import Control.Monad (unless, liftM)
import Data.Array (Array, array, (//), (!))
import Data.Bits (testBit)
import Data.Map (Map)
import qualified Data.Map as Map (empty, lookup, insert, delete)
import Data.List (genericTake, genericDrop, genericSplitAt, genericReplicate,
                  genericLength,
                  mapAccumL, sort)
import Data.Word (Word8, Word16, Word32)
\end{code}

\section{Types}

Type synonyms are your friend.

\begin{code}
type Octet = Word8     -- The inut/output type
type Octets = [Octet]

type Code = Word16     -- A generic code
type Dist = Code       -- A distance code (1-32768)
type LitLen = Code     -- A literal/length code (3-258)
type Length = Word8    -- Number of bits needed to identify a code

type Table = Map Code Bits
type Tables = (Table, Table) -- lit/len and dist Huffman tables
\end{code}

The \verb!Bit! datatype is used for the input. We can show values and
convert to and from octet streams and Word32s.

\begin{code}
newtype Bit = Bit Bool
    deriving Eq
type Bits = [Bit]
instance Show Bit where
    show = (\x -> [x]) . show_b
    showList bs = showString $ "'" ++ map show_b bs ++ "'"

show_b :: Bit -> Char
show_b (Bit True) = '1'
show_b (Bit False) = '0'

octet_to_bits :: Octet -> Bits
octet_to_bits n = map (\i -> Bit (testBit n i)) [0..7]

word16_to_bits :: Word16 -> Bits
word16_to_bits n = map (\i -> Bit (testBit n i)) [0..15]

bits_to_octets :: Bits -> [Octet]
bits_to_octets [] = []
bits_to_octets xs = case splitAt 8 xs of
                        (bs, xs') -> bits_to_octet bs:bits_to_octets xs'

bits_to_octet :: Bits -> Octet
bits_to_octet = foldr (\(Bit b) i -> 2 * i + (if b then 1 else 0)) 0
\end{code}

\section{Monad}

We carry around the input bits remaining, how many we have used mod 8
(so that we can octet-align), the last 32768 output octets in an array
so we can repeat them and where in the (cyclic) array we are. The offset
is rarely used, so we make it strict to avoid building huge closures.

\begin{code}
data State = State { octets :: [Octet],             -- remaining input octets
                     offset :: !Int,                -- num bits output mod 8
                     history :: Array Word16 Octet, -- last 32768 output words
                     loc :: Word16,                 -- where in history we are
                     -- Find list oflocs in history that begin with 3 octets
                     backrefs :: Map (Octet, Octet, Octet) [Word16],
                     once_round :: Bool -- Have we gone round the array once?
                   }
data DefM a = DefM (State -> (a, State))

instance Monad DefM where
 -- (>>=)  :: DefM a -> (a -> DefM b) -> DefM b
    DefM v >>= f = DefM $ \s -> let (x, s') = v s
                                    DefM y = f x
                                in y s'
 -- return :: a -> DefM a
    return x = DefM $ \s -> (x, s)

extract_DefM :: DefM a -> a
extract_DefM (DefM f) = let (x, s) = f undefined in x

set_octets :: Octets -> DefM ()
set_octets os = DefM $ const ((), State {octets = os,
                                         offset = 0,
                                         history = array (0, 32767) [],
                                         loc = 0,
                                         backrefs = Map.empty,
                                         once_round = False})
\end{code}

We also provide various helper functions to align the remaining bits to
an octet boundary and various functions to get some bits from the input.

\begin{code}
{-
no_bits :: DefM Bool
no_bits = DefM $ \s -> (null (bits s), s)
-}

align_8_bits :: DefM Bits
align_8_bits
 = DefM $ \s -> (replicate ((8 - offset s) `mod` 8) (Bit False),
                 s { offset = 0 })

gen_bs :: Int -> DefM ()
gen_bs n
 = DefM $ \s -> ((), s { offset = (offset s + n) `mod` 8 })

lookup_triple :: (Octet, Octet, Octet) -> DefM [Word16]
lookup_triple os = DefM $ \s -> case Map.lookup os (backrefs s) of
                                    Nothing -> ([], s)
                                    Just is -> (is, s)

get_octet :: DefM (Maybe Octet)
get_octet = DefM $ \s -> case octets s of
                             [] -> (Nothing, s)
                             (o:os) -> (Just o, s { octets = os })

drop_input :: Word16 -> DefM ()
drop_input n = DefM $ \s -> ((), s { octets = genericDrop n (octets s) })

get_input :: DefM [Octet]
get_input = DefM $ \s -> (octets s, s)

get_back :: Word16 -> Word16 -> DefM [Octet]
get_back dist n = do l <- get_loc
                     get_from ((l - dist) `mod` 32768) n

get_from :: Word16 -> Word16 -> DefM [Octet]
get_from x n = DefM $ \s ->
                let l = loc s
                    h = history s
                    stub = map (h!)
                         $ genericTake (32768 - ((x - l) `mod` 32768))
                                       ([x `mod` 32768..32767] ++ [0..])
                    res = genericTake n (cycle stub)
                in (res, s)

get_loc :: DefM Word16
get_loc = DefM $ \s -> (loc s, s)
\end{code}

We have 2 ways to provide more output. We can either write a single
octet out or repeat a given number of bits a given distance back in the
history.

\begin{code}
output :: Octet -> DefM ()
output w
 = DefM $ \s -> let l = loc s
                    l' = l + 1 `mod` 32768
                    l'' = l' + 1 `mod` 32768
                    l_ = l - 1 `mod` 32768
                    l__ = l' - 1 `mod` 32768
                    old_triple = (h ! l, h ! l', h ! l'')
                    new_triple = (h' ! l__, h' ! l_, h' ! l)
                    o = once_round s
                    h = history s
                    h' = h // [(l, w)]
                    b = backrefs s
                    b' = case Map.lookup new_triple b of
                             Nothing -> Map.insert new_triple [l__]    b
                             Just ls -> Map.insert new_triple (l__:ls) b
                    b'' = case Map.lookup old_triple b' of
                              Nothing -> error "output: Can't happen"
                              Just [_] -> Map.delete old_triple b' -- XXX Could check this and next -- l
                              Just ls -> Map.insert old_triple (init ls) b'
                              -- XXX init inefficient
                in if o then ((), s { history = h',
                                      loc = l',
                                      backrefs = b'' })
                   else if l == 32767 then ((), s { history = h',
                                                    loc = l',
                                                    backrefs = b',
                                                    once_round = True })
                   else if l > 1 then ((), s { history = h',
                                               loc = l',
                                               backrefs = b' })
                   else ((), s { history = h', loc = l' })

repeat_w32s :: Word16 -> Word16 -> DefM ()
repeat_w32s len dist = do os <- get_back dist len
                          mapM_ output os -- XXX Horribly inefficient?
\end{code}

\section{Deflate itself}

The hardcore stuff!

To inflate an octet stream we use inflate\_blocks to do the hard work.
It in turn looks at the first 3 bits to decide whether to just output an
uncompressed segment or pass off the work to inflate\_tables and
inflate\_codes.

\begin{code}
deflate :: Octets -> Octets
deflate os = extract_DefM $ do set_octets os
                               bs <- deflate_block
                               bs' <- align_8_bits
                               return (bits_to_octets (bs ++ bs'))
    where len = genericLength os :: Word16
          nlen = -1 - len

d :: Octets -> Bits
d os = extract_DefM $ do set_octets os
                         bs <- deflate_block
                         bs' <- align_8_bits
                         return (bs ++ bs')
    where len = genericLength os :: Word16
          nlen = -1 - len

-- We are guaranteed length >= 2
deflate_block :: DefM Bits
deflate_block
     = do let header = [Bit True, -- is last
                        Bit True, Bit False] -- Compress with fixed tables
              footer = lookup_tab (fst inflate_trees_fixed) 256
          gen_bs 3
          the_data <- deflate_octets inflate_trees_fixed
          gen_bs (length footer)
          return (header ++ the_data ++ footer)
\end{code}

\begin{code}
lookup_tab :: Table -> Code -> Bits
lookup_tab t c = case Map.lookup c t of
                     Nothing -> error "lookup_tab: Didn't find answer"
                     Just bs -> reverse bs

deflate_octets :: Tables -> DefM Bits
deflate_octets tabs@(tab_litlen, tab_dist)
 = do mo1 <- get_octet
      mo2 <- get_octet
      case mo1 of
          Nothing -> return []
          Just o1 ->
              case mo2 of
                  Nothing ->
                      do let bs = lookup_tab tab_litlen (fromIntegral o1)
                         gen_bs (length bs)
                         output o1
                         return bs
                  Just o2 -> deflate_octets' tabs o1 o2

get_dist_length :: [Word16] -> DefM (Dist, LitLen)
get_dist_length indexes
 = do input <- get_input
      l <- get_loc
      let f :: (Dist, LitLen) -> [Word16] -> DefM (Dist, LitLen)
          f res [] = return res
          f res@(_, 255) _ = return res
          f res@(best_dist, best_len) (i:is)
           = do inp <- get_from i 255
                let my_len = 3 + (genericLength
                                $ takeWhile id $ zipWith (==) input inp)
                    res' = if my_len > best_len
                           then (32768 - ((i - l) `mod` 32770), my_len)
                           else res
                f res' is
      (dist, len) <- f (0, 0) indexes
      drop_input (len - 3)
      return (dist, len)

deflate_octets' :: Tables -> Octet -> Octet -> DefM Bits
deflate_octets' tabs@(tab_litlen, tab_dist) o1 o2
 = do mo3 <- get_octet
      case mo3 of
          Nothing -> do output o1
                        output o2
                        let bs1 = lookup_tab tab_litlen (fromIntegral o1)
                            bs2 = lookup_tab tab_litlen (fromIntegral o2)
                            bs = bs1 ++ bs2
                        gen_bs (length bs)
                        return bs
          Just o3 -> do is <- lookup_triple (o1, o2, o3)
                        case is of
                            [] -> do let this = lookup_tab tab_litlen
                                                           (fromIntegral o1)
                                     output o1
                                     bs <- deflate_octets' tabs o2 o3
                                     return (this ++ bs)
                            _ -> do (dist, len) <- get_dist_length is
                                    let bs_len = get_len_bs tab_litlen len
                                    let bs_dist = get_dist_bs tab_dist dist
                                    gen_bs $ length bs_len
                                    gen_bs $ length bs_dist
                                    repeat_w32s len dist
                                    bs <- deflate_octets tabs
                                    return (bs_len ++ bs_dist ++ bs)
\end{code}

\begin{code}
mk_bs :: Int -> Word16 -> Bits
mk_bs n x = map (\i -> Bit (testBit x i)) [0..n-1]
{-
mk_bs :: Int -> Word16 -> Bits
mk_bs n x = map (\i -> Bit (testBit x i)) (reverse [0..n-1])
-}

get_len_bs :: Table -> LitLen -> Bits
get_len_bs tab len = f litlens
    where f [] = error $ "get_len_bs: Can't happen: " ++ show len
          f ((ll, (l, n)):xs)
           | l <= len = lookup_tab tab ll ++ mk_bs n (len - ll)
           | otherwise = f xs

litlens :: [(LitLen, (LitLen, Int))]
litlens = zip [285,284..] [(258, 0),
                           (227, 5),
                           (195, 5),
                           (163, 5),
                           (131, 5),
                           (115, 4),
                           ( 99, 4),
                           ( 83, 4),
                           ( 67, 4),
                           ( 59, 3),
                           ( 51, 3),
                           ( 43, 3),
                           ( 35, 3),
                           ( 31, 2),
                           ( 27, 2),
                           ( 23, 2),
                           ( 19, 2),
                           ( 17, 1),
                           ( 15, 1),
                           ( 13, 1),
                           ( 11, 1),
                           ( 10, 0),
                           (  9, 0),
                           (  8, 0),
                           (  7, 0),
                           (  6, 0),
                           (  5, 0),
                           (  4, 0),
                           (  3, 0)]

get_dist_bs :: Table -> Dist -> Bits
get_dist_bs tab dist = f dists
    where f [] = error "get_dist_bs: Can't happen"
          f ((ll, (l, n)):xs)
           | l <= dist = lookup_tab tab ll ++ mk_bs n (dist - ll)
           | otherwise = f xs

dists :: [(Dist, (Word16, Int))]
dists = zip [29,28..] [(24577, 13),
                       (16385, 13),
                       (12289, 12),
                       ( 8193, 12),
                       ( 6145, 11),
                       ( 4097, 11),
                       ( 3073, 10),
                       ( 2049, 10),
                       ( 1537,  9),
                       ( 1025,  9),
                       (  769,  8),
                       (  513,  8),
                       (  385,  7),
                       (  257,  7),
                       (  193,  6),
                       (  129,  6),
                       (   97,  5),
                       (   65,  5),
                       (   49,  4),
                       (   33,  4),
                       (   25,  3),
                       (   17,  3),
                       (   13,  2),
                       (    9,  2),
                       (    7,  1),
                       (    5,  1),
                       (    4,  0),
                       (    3,  0),
                       (    2,  0),
                       (    1,  0)]
\end{code}

\section{Fixed tables}

The fixed tables. Not much to say really.

\begin{code}
inflate_trees_fixed :: Tables
inflate_trees_fixed = (make_table $ [(8, c) | c <- [0..143]]
                                 ++ [(9, c) | c <- [144..255]]
                                 ++ [(7, c) | c <- [256..279]]
                                 ++ [(8, c) | c <- [280..287]],
                       make_table [(5, c) | c <- [0..29]])
\end{code}

\section{The Huffman Tree}

\begin{code}
make_table :: [(Length, Code)] -> Table
make_table lcs
 = case make_table' Map.empty 0 [] $ sort $ filter ((/= 0) . fst) lcs of
       (table, []) -> table
       _ -> error $ "make_table: Left-over lcs"

make_table' :: Table -> Length -> Bits -> [(Length, Code)]
            -> (Table, [(Length, Code)])
make_table' table _ _ [] = (table, [])
make_table' table i bs lcs@((l, c):lcs')
 | i == l = (Map.insert c bs table, lcs')
 | i < l = let (table', lcs') = make_table' table  (i+1) (Bit False:bs) lcs
           in                   make_table' table' (i+1) (Bit True:bs)  lcs'
 | otherwise = error "make_tree: can't happen"
\end{code}

